var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { compOperators, compOpsAndSymbols, compOpToSelector, compOpToSymbol, } from "../helpers/operators";
import { escapeRegexChars } from "../helpers/strings";
import { KeyValueType } from "./KeyValueType";
var NumberKeyValueType = /** @class */ (function (_super) {
    __extends(NumberKeyValueType, _super);
    function NumberKeyValueType() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._delimiters = {
            decimal: ".",
            groups: ",",
        };
        _this._groupsDelimiterRegex = /,/gi;
        return _this;
    }
    NumberKeyValueType.mongoSelectorFor = function (value, numberTransform) {
        var _a;
        if (numberTransform === void 0) { numberTransform = function (n) { return n; }; }
        return _a = {}, _a[compOpToSelector(value.operator)] = numberTransform(value.number), _a;
    };
    Object.defineProperty(NumberKeyValueType.prototype, "delimiters", {
        set: function (delimiters) {
            this._delimiters = delimiters;
            this._groupsDelimiterRegex = new RegExp(escapeRegexChars(this._delimiters.groups), "gi");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NumberKeyValueType.prototype, "numberFormatter", {
        set: function (numberFormatter) {
            this._numberFormatter = numberFormatter;
        },
        enumerable: true,
        configurable: true
    });
    NumberKeyValueType.prototype.getSuggestions = function (prefix) {
        var _this = this;
        var parsed = this.parseString(prefix);
        // empty input or invalid input or fallback "="
        if (prefix === "" || !parsed || (parsed.operator === "=" && !prefix.startsWith("="))) {
            var number_1 = 42;
            var operators = Object.keys(compOperators);
            if (prefix !== "") {
                number_1 = parsed ? parsed.number : number_1;
                var filteredOperators = compOpsAndSymbols.filter(function (op) { return op.startsWith(prefix); });
                operators = filteredOperators.length > 0 ? filteredOperators : operators;
            }
            return operators.map(function (operator) { return ({
                display: _this.display({ operator: operator, number: number_1 }),
                value: _this.editText({ operator: operator, number: number_1 }),
            }); });
        }
        else {
            return [];
        }
    };
    NumberKeyValueType.prototype.parseString = function (val) {
        var _a;
        var value = val.trim();
        if (value === "") {
            value = ">0";
        }
        value = value.replace(/\s/g, "") // remove spaces
            .replace(this._groupsDelimiterRegex, "") // remove grouping delimiters
            .replace(this._delimiters.decimal, "."); // convert to decimal point
        var match = value.replace("≥", ">=")
            .replace("≤", "<=")
            .replace("<>", "!=")
            .replace("≠", "!=")
            .replace("==", "=")
            .match(/^(<=|<|>=|>|!=|=)?(-?\d*\.?\d+)/);
        if (match) {
            return {
                operator: (_a = match[1], (_a !== null && _a !== void 0 ? _a : "=")),
                number: Number(match[2]),
            };
        }
        else {
            return undefined;
        }
    };
    NumberKeyValueType.prototype.checkValue = function (val) {
        if (Object.keys(compOperators).includes(val.operator) && typeof val.number === "number") {
            return val;
        }
        else {
            return undefined;
        }
    };
    NumberKeyValueType.prototype.display = function (value) {
        var numberString = this._numberFormatter ? this._numberFormatter.format(value.number) : String(value.number);
        return { text: compOpToSymbol(value.operator) + " " + numberString };
    };
    NumberKeyValueType.prototype.editText = function (value) {
        return this.display(value).text;
    };
    NumberKeyValueType.prototype.isValueEqual = function (valueA, valueB) {
        if (_super.prototype.isValueEqual.call(this, valueA, valueB)) {
            return true;
        }
        if (valueA && valueB) {
            return valueA.operator === valueB.operator && valueA.number === valueB.number;
        }
        return false;
    };
    return NumberKeyValueType;
}(KeyValueType));
export default NumberKeyValueType;
